#ifndef VSSTRING_H
#define VSSTRING_H

class StringFile;

class String
{
	friend StringFile;

private:
	enum
	{
		VS_DefaultLength = 128
	};

	static int queryCharsize(void);
	static int charsz;
	static int numStringsAllocated;  // count of all strings which have hit their constructor but NOT their destructor (still allocated)

	String(const String &);      // disable copy constructor

protected:
	int length;      // in character units
	int allocSize;   // in bytes
	u_int8 *str;

public:  
	// static members
	static void install(int charsize);
	static void remove(void);

	static int  strlen(const void *str,int charsize=charsz);
	static int  strlen(const String &vstr,int charsize=charsz);
	static void strcpy(void *dest,const void *src,int srcLength=-1,int charsize=charsz);

	// dynamic members
	explicit String(const void *initString,int charsize=charsz);
	explicit String(int allocsize=VS_DefaultLength);
	~String(void);

	int         getLength(void) const;
	const void *getPointer(int index=0) const;
	void        setNull(int index=0);
	int         getChar(int index) const;

	void append(const void *appendString,int charsize=charsz);
	void append(const String &appendString);
	void append(int appendChar);

	int           operator[](int index) const;
	bool          operator==(const String &src) const;
	bool          operator!=(const String &src) const;
	String       &operator+=(const String &src);
	String       &operator=(const String &src);
};

//------------------------------------------------------

inline int String::queryCharsize(void)
{
	return charsz;
}

inline int String::strlen(const String &vstr,int charsize)
{
	return strlen(vstr.str,charsize);
}

//------------------------------------------------------

inline int String::getLength(void) const
{
	return length;
}

inline const void *String::getPointer(int index) const
{
	return str+index*charsz;
}

inline void String::append(const String &appendString)
{
	append(appendString.str);
}

inline void String::setNull(int index)
{
	if (index<=length)
	{
		length=index;
		*(reinterpret_cast<u_int32 *>(str+index*charsz))=0;  // plop a 4-byte wide null there to cover all cases
	}
}

inline int String::getChar(int index) const
{
	return operator[](index);
}

//------------------------------------------------------

inline String &String::operator+=(const String &src)
{
	append(src.str);
	return *this;
}

inline bool String::operator!=(const String &src) const
{
	return !(operator==(src));
}

#endif
